@@ -23,27 +23,8 @@ def _sqlite_is_bad_expr(dbmodel, expression):
     )
 
 
-def _sqlite_mean_expr(dbmodel, expression):
-    return (
-        "avg(" + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False) + ")"
-    )
-
-
-def _sqlite_lag_expr(dbmodel, expression):
-    return (
-        "LAG(" + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False) + ")"
-    )
-
-
-def _sqlite_size_expr(dbmodel, expression):
-    return "SUM(1)"
-
-
 SQLite_formatters = {
     "is_bad": _sqlite_is_bad_expr,
-    "mean": _sqlite_mean_expr,
-    "shift": _sqlite_lag_expr,
-    "size": _sqlite_size_expr,
 }
 
 
@@ -77,6 +58,7 @@ class SQLiteModel(data_algebra.db_model.DBModel):
             identifier_quote='"',
             string_quote="'",
             sql_formatters=SQLite_formatters,
+            on_joiner='AND'
         )
 
     def prepare_connection(self, conn):
@@ -92,6 +74,7 @@ class SQLiteModel(data_algebra.db_model.DBModel):
             "atan": math.atan,
             "atanh": math.atanh,
             "ceil": math.ceil,
+            "ceiling": math.ceil,
             "cos": math.cos,
             "cosh": math.cosh,
             "degrees": math.degrees,
@@ -114,6 +97,7 @@ class SQLiteModel(data_algebra.db_model.DBModel):
             "log2": math.log2,
             "modf": math.modf,
             "radians": math.radians,
+            "sign": numpy.sign,
             "sin": math.sin,
             "sinh": math.sinh,
             "sqrt": math.sqrt,
@@ -122,7 +106,7 @@ class SQLiteModel(data_algebra.db_model.DBModel):
             "trunc": math.trunc,
         }
         for k, f in math_fns.items():
-            if not k in saw:
+            if k not in saw:
                 conn.create_function(k, 1, f)
                 saw.add(k)
         math_fns_2 = {
@@ -134,9 +118,10 @@ class SQLiteModel(data_algebra.db_model.DBModel):
             "isclose": math.isclose,
             "ldexp": math.ldexp,
             "pow": math.pow,
+            "power": math.pow,
         }
         for k, f in math_fns_2.items():
-            if not k in saw:
+            if k not in saw:
                 conn.create_function(k, 2, f)
                 saw.add(k)
         # numpy fns
@@ -163,12 +148,12 @@ class SQLiteModel(data_algebra.db_model.DBModel):
             "tanh": numpy.tanh,
         }
         for k, f in numpy_fns.items():
-            if not k in saw:
+            if k not in saw:
                 conn.create_function(k, 1, f)
                 saw.add(k)
 
         # https://docs.python.org/3/library/sqlite3.html
-        conn.create_aggregate('median', 1, MedianAgg)
+        conn.create_aggregate("median", 1, MedianAgg)
 
     # noinspection PyMethodMayBeStatic,SqlNoDataSourceInspection
     def insert_table(
@@ -208,8 +193,6 @@ def example_handle():
     Return an example db handle for testing. Returns None if helper packages not present.
 
     """
-    db_handle = SQLiteModel().db_handle(
-        conn=sqlite3.connect(':memory:')
-    )
+    db_handle = SQLiteModel().db_handle(conn=sqlite3.connect(":memory:"))
     db_handle.db_model.prepare_connection(db_handle.conn)
     return db_handle
