@@ -107,6 +107,20 @@ def _k_or(*args):
     return res
 
 
+def _k_add(*args):
+    res = args[0]
+    for i in range(1, len(args)):
+        res = numpy.add(res, args[i])
+    return res
+
+
+def _k_mul(*args):
+    res = args[0]
+    for i in range(1, len(args)):
+        res = numpy.multiply(res, args[i])
+    return res
+
+
 def populate_impl_map(data_model):
     impl_map = {
         "==": _type_safe_equal,
@@ -117,10 +131,10 @@ def populate_impl_map(data_model):
         "<=": numpy.less_equal,  # already checks types
         ">": numpy.greater,  # already checks types
         ">=": numpy.greater_equal,  # already checks types
-        "+": numpy.add,
+        "+": _k_add,
         "-": negate_or_subtract,
         "neg": numpy.negative,
-        "*": numpy.multiply,
+        "*": _k_mul,
         "/": numpy.divide,
         "//": numpy.floor_divide,
         "%": numpy.mod,
@@ -140,7 +154,7 @@ def populate_impl_map(data_model):
         "is_bad": data_model.bad_column_positions,
         "is_in": _type_safe_is_in,
         "concat": lambda a, b: numpy.char.add(
-            numpy.asarray(a, dtype=numpy.str), numpy.asarray(b, dtype=numpy.str)
+            numpy.asarray(a, dtype=str), numpy.asarray(b, dtype=str)
         ),
         "coalesce": lambda a, b: _coalesce(a, b),  # assuming Pandas series
         "connected_components": lambda a, b: data_algebra.connected_components.connected_components(
@@ -245,14 +259,6 @@ class DataModel(ABC):
     def extend_step(self, op, *, data_map, narrow):
         raise NotImplementedError("base method called")
 
-    def columns_to_frame(self, cols):
-        """
-
-        :param cols: dictionary mapping column names to columns
-        :return:
-        """
-        raise NotImplementedError("base method called")
-
     def project_step(self, op, *, data_map, narrow):
         raise NotImplementedError("base method called")
 
