@@ -79,7 +79,29 @@ class ViewRepresentation(OperatorPlatform, ABC):
         )
 
     def merged_rep_id(self):
-        return "node+ " + str(id(self))
+        return "ops+ " + str(id(self))
+
+    # convenience
+
+    def ex(self, *, data_model=None, narrow=True, allow_limited_tables=False):
+        """
+        Evaluate operators with respect to Pandas data frames already stored in the operator chain.
+
+        :param data_model: adaptor to data dialect (Pandas for now)
+        :param narrow: logical, if True don't copy unexpected columns
+        :param allow_limited_tables: logical, if True allow execution on non-complete tables
+        :return: table result
+        """
+        tables = self.get_tables()
+        for t in tables.values():
+            assert t.head is not None
+            if len(tables) > 1:
+                assert t.table_name_was_set_by_user
+            if not allow_limited_tables:
+                assert t.limit_was is None
+                assert t.nrows == t.head.shape[0]
+        data_map = {k: v.head for k, v in tables.items()}
+        return self.eval(data_map = data_map, data_model=data_model, narrow=narrow)
 
     # characterization
 
@@ -91,7 +113,8 @@ class ViewRepresentation(OperatorPlatform, ABC):
             s = self.sources[i]
             ti = s.get_tables()
             for (k, v) in ti.items():
-                assert isinstance(v, TableDescription)
+                assert isinstance(v, OperatorPlatform)
+                assert v.node_name == "TableDescription"
                 if k in tables.keys():
                     if not v.same_table(tables[k]):
                         raise ValueError(
@@ -102,7 +125,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
         return tables
 
     def columns_used_from_sources(self, using=None):
-        """Get column names used from direct source nodes when this node is exececuted
+        """Get column names used from direct source nodes when this ops is exececuted
         with the using columns (None means all)."""
         raise NotImplementedError("base method called")
 
@@ -190,7 +213,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
 
     # noinspection PyMethodMayBeStatic,PyUnusedLocal
     def _equiv_nodes(self, other):
-        """Check if immediate node structure is equivalent, does not check child nodes"""
+        """Check if immediate ops structure is equivalent, does not check child nodes"""
         raise NotImplementedError("base method called")
 
     def __eq__(self, other):
@@ -277,10 +300,11 @@ class ViewRepresentation(OperatorPlatform, ABC):
     def eval(self, data_map, *, data_model=None, narrow=True):
         """
          Evaluate operators with respect to Pandas data frames.
+
          :param data_map: map from table names to data frames
          :param data_model: adaptor to data dialect (Pandas for now)
          :param narrow logical, if True don't copy unexpected columns
-         :return:
+         :return: table result
          """
 
         assert isinstance(data_map, dict)
@@ -306,6 +330,13 @@ class ViewRepresentation(OperatorPlatform, ABC):
 
     # noinspection PyPep8Naming
     def transform(self, X, *, data_model=None, narrow=True):
+        """
+        Apply data transfrom to a table
+
+        :param X: tale to apply to
+        :param data_model: data model for Pandas execution
+        :param narrow: logical, if True narrow number of result columns to specification
+        """
         if data_model is None:
             data_model = data_algebra.default_data_model
         assert isinstance(data_model, data_algebra.data_model.DataModel)
@@ -338,7 +369,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
             column_types=column_types,
         )
 
-    # implement builders for all non-initial node types on base class
+    # implement builders for all non-initial ops types on base class
     def extend_parsed(
         self, parsed_ops, *, partition_by=None, order_by=None, reverse=None
     ):
@@ -392,7 +423,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
                         order_by=order_by,
                         reverse=reverse,
                     )
-        # new node
+        # new ops
         return ExtendNode(
             source=self,
             parsed_ops=parsed_ops,
@@ -432,7 +463,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
         parsed_ops = data_algebra.expr_parse.parse_assignments_in_context(ops, self)
         return self.project_parsed(parsed_ops=parsed_ops, group_by=group_by)
 
-    def natural_join(self, b, *, by, jointype):
+    def natural_join(self, b, *, by, jointype, check_all_common_keys_in_by=False):
         """
 
         :param b: right table
@@ -443,7 +474,12 @@ class ViewRepresentation(OperatorPlatform, ABC):
         assert isinstance(b, ViewRepresentation)
         if self.is_trivial_when_intermediate():
             return self.sources[0].natural_join(b, by=by, jointype=jointype)
-        return NaturalJoinNode(a=self, b=b, by=by, jointype=jointype)
+        return NaturalJoinNode(
+            a=self,
+            b=b,
+            by=by,
+            jointype=jointype,
+            check_all_common_keys_in_by=check_all_common_keys_in_by)
 
     def concat_rows(self, b, *, id_column="source_name", a_name="a", b_name="b"):
         if b is None:
@@ -467,6 +503,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
     def select_rows(self, expr):
         if expr is None:
             return self
+        assert isinstance(expr, (str, data_algebra.expr_rep.PreTerm))
         if self.is_trivial_when_intermediate():
             return self.sources[0].select_rows(expr)
         ops = data_algebra.expr_parse.parse_assignments_in_context({"expr": expr}, self)
@@ -525,7 +562,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
         )
 
 
-# Could also have general query as starting node, but don't see a lot of point to
+# Could also have general query as starting ops, but don't see a lot of point to
 # it until somebody needs it.
 
 
@@ -542,11 +579,12 @@ class TableDescription(ViewRepresentation):
     column_names: List[str]
     qualifiers: Dict[str, str]
     key: str
+    table_name_was_set_by_user: bool
 
     def __init__(
         self,
         *,
-        table_name,
+        table_name=None,
         column_names,
         qualifiers=None,
         sql_meta=None,
@@ -559,9 +597,11 @@ class TableDescription(ViewRepresentation):
             self, column_names=column_names, node_name="TableDescription"
         )
         if table_name is None:
-            table_name = ""
-        if table_name is not None:
-            assert isinstance(table_name, str)
+            self.table_name_was_set_by_user = False
+            table_name = "data_frame"
+        else:
+            self.table_name_was_set_by_user = True
+        assert isinstance(table_name, str)
         if head is not None:
             if set([c for c in head.columns]) != set(column_names):
                 raise ValueError("head.columns != column_names")
@@ -708,21 +748,23 @@ class TableDescription(ViewRepresentation):
 
 def describe_table(
     d,
-    table_name="data_frame",
+    table_name=None,
     *,
     qualifiers=None,
     sql_meta=None,
     column_types=None,
     row_limit=7,
-    keep_sample=True
+    keep_sample=True,
+    keep_all=False
 ):
     """
     :param d: pandas table to describe
     :param table_name: name of table
     :param qualifiers: optional able qualifiers
     :param sql_meta: optional sql meta information map
-    :param row_limit: how many rows to sampe
+    :param row_limit: how many rows to sample
     :param keep_sample: logical, if True retain head of table
+    :param keep_all: logical, if True retain all of table
     :return: TableDescription
     """
     assert d is not None
@@ -732,9 +774,12 @@ def describe_table(
     if column_types is None:
         column_types = data_algebra.util.guess_column_types(d)
     head = None
-    nrows = None
-    if keep_sample:
-        nrows = d.shape[0]
+    nrows = d.shape[0]
+    if keep_all:
+        row_limit = None
+        head = d.copy()
+        head.reset_index(drop=True, inplace=True)
+    elif keep_sample:
         if nrows > row_limit:
             head = d.iloc[range(row_limit), :].copy()
         else:
@@ -752,6 +797,26 @@ def describe_table(
     )
 
 
+def table(d, table_name):
+    """
+    Capture a table for later use
+
+    :param d: Pandas data frame to capture
+    :param table_name: name for this table
+    :return: a table description
+    """
+    return describe_table(
+        d=d,
+        table_name=table_name,
+        qualifiers=None,
+        sql_meta=None,
+        column_types=None,
+        row_limit=None,
+        keep_sample=True,
+        keep_all=True,
+    )
+
+
 class ExtendNode(ViewRepresentation):
     def __init__(
         self, *, source, parsed_ops, partition_by=None, order_by=None, reverse=None
@@ -1499,7 +1564,7 @@ class NaturalJoinNode(ViewRepresentation):
     by: List[str]
     jointype: str
 
-    def __init__(self, a, b, *, by, jointype):
+    def __init__(self, a, b, *, by, jointype, check_all_common_keys_in_by=False):
         # check set of tables is consistent in both sub-dags
         a_tables = a.get_tables()
         b_tables = b.get_tables()
@@ -1507,8 +1572,8 @@ class NaturalJoinNode(ViewRepresentation):
             raise ValueError(
                 "Must specify by in natural joins ([] for empty conditions)"
             )
-        common_keys = set(a_tables.keys()).intersection(b_tables.keys())
-        for k in common_keys:
+        common_table_keys = set(a_tables.keys()).intersection(b_tables.keys())
+        for k in common_table_keys:
             if not a_tables[k].same_table(b_tables[k]):
                 raise ValueError(
                     "Different definition of table object on a/b for: " + k
@@ -1530,6 +1595,12 @@ class NaturalJoinNode(ViewRepresentation):
         missing_right = by_set - b.column_set
         if len(missing_right) > 0:
             raise KeyError("right table missing join keys: " + str(missing_right))
+        if check_all_common_keys_in_by:
+            missing_common = a.column_set.intersection(b.column_set) - by_set
+            if len(missing_common) > 0:
+                raise KeyError(
+                    "check_all_common_keys_in_by set, and the following common keys are are not in the by-clause: "
+                    + str(missing_common))
         ViewRepresentation.__init__(
             self,
             column_names=column_names,
