@@ -16,7 +16,7 @@ class EvalModel(ABC):
             new_name = "TMP_" + str(new_id).zfill(7) + "_T"
         return new_name
 
-    def eval(self, ops, *, data_map=None, result_name=None, narrow=True):
+    def managed_eval(self, ops, *, data_map=None, result_name=None, narrow=True):
         """
         apply ops to data frames in data_map
 
