@@ -84,6 +84,65 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
                 assert data_algebra.util.compatible_types({types_seen[c]}.union({op.column_types[c]}))
         return res
 
+    def columns_to_frame_(self, cols, *, target_rows=0):
+        """
+        Convert a dictionary of column names to series-like objects and scalars into a Pandas data frame.
+
+        :param cols: dictionary mapping column names to columns
+        :return: Pandas data frame.
+        """
+        # noinspection PyUnresolvedReferences
+        assert isinstance(cols, dict)
+        if len(cols) < 1:
+            return self.pd.DataFrame(cols)
+        for k, v in cols.items():
+            try:
+                target_rows = max(target_rows, len(v))
+            except TypeError:
+                target_rows = max(target_rows, 1)  # scalar
+        if target_rows < 1:
+            return self.pd.DataFrame(cols)
+
+        # agg can return scalars, which then can't be made into a self.pd.DataFrame
+        def promote_scalar(v, *, target_len):
+            # noinspection PyBroadException
+            try:
+                len_v = len(v)
+                if len_v != target_len:
+                    if len_v == 0:
+                        return [None] * target_len
+                    elif len_v == 1:
+                        return [v[0]] * target_len
+                    else:
+                        raise ValueError("incompatible column lengths")
+            except Exception:
+                return [v] * target_len  # scalar
+            return v
+
+        cols = {k: promote_scalar(v, target_len=target_rows) for (k, v) in cols.items()}
+        return self.pd.DataFrame(cols)
+
+    def add_data_frame_columns_to_data_frame_(self, res, transient_new_frame):
+        if (res.shape[0] == 0) and (transient_new_frame.shape[0] > 0):
+            # scalars get interpreted as single row items, instead of zero row items
+            # growing the extension frame
+            transient_new_frame = transient_new_frame.iloc[range(0), :].reset_index(drop=True)
+        if res.shape[0] == transient_new_frame.shape[0]:
+            if res.shape[1] < 1:
+                return transient_new_frame
+            if transient_new_frame.shape[1] < 1:
+                return res
+        if (2 * transient_new_frame.shape[1]) > res.shape[1]:
+            # lots of columns path
+            # https://win-vector.com/2021/08/03/i-think-pandas-may-have-lost-the-plot/
+            for c in set(res.columns).intersection(set(transient_new_frame.columns)):
+                del res[c]
+            return self.pd.concat([res, transient_new_frame], axis=1)
+        # normal path
+        for c in transient_new_frame.columns:
+           res[c] = transient_new_frame[c]
+        return res
+
     def extend_step(self, op, *, data_map, narrow):
         if op.node_name != "ExtendNode":
             raise TypeError("op was supposed to be a data_algebra.data_ops.ExtendNode")
@@ -97,12 +156,14 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         res = op.sources[0].eval_implementation(
             data_map=data_map, data_model=self, narrow=narrow
         )
-        data_algebra_temp_cols = {}
         if not window_situation:
-            for (k, opk) in op.ops.items():
-                res_k = opk.evaluate(res)  # makes debugging easier
-                res[k] = res_k
+            new_cols = {k: opk.evaluate(res) for k, opk in op.ops.items()}
+            # for k, v in new_cols.items():
+            #     res[k] = v
+            new_frame = self.columns_to_frame_(new_cols, target_rows=res.shape[0])
+            res = self.add_data_frame_columns_to_data_frame_(res, new_frame)
         else:
+            data_algebra_temp_cols = {}
             standin_name = "_data_algebra_temp_g"  # name of an arbitrary input variable
             # build up a sub-frame to work on
             col_list = [c for c in set(op.partition_by)]
@@ -186,32 +247,12 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             for value_name in data_algebra_temp_cols.values():
                 del res[value_name]
             # copy out results
-            subframe = subframe.reset_index(drop=True)
             subframe = subframe.sort_values(by=["_data_algebra_orig_index"])
+            subframe = subframe.loc[:, list(op.ops.keys())]
             subframe = subframe.reset_index(drop=True)
-            for (k, opk) in op.ops.items():
-                res[k] = subframe[k]
+            res = self.add_data_frame_columns_to_data_frame_(res, subframe)
         return res
 
-    def columns_to_frame(self, cols):
-        """
-
-        :param cols: dictionary mapping column names to columns
-        :return:
-        """
-        # noinspection PyUnresolvedReferences
-        assert isinstance(cols, dict)
-        # check columns all have the same length
-        if len(cols) > 1:
-            len_0 = None
-            for v in cols.items():
-                len_i = len(v)
-                if len_0 is None:
-                    len_0 = len_i
-                else:
-                    assert len_i == len_0
-        return self.pd.DataFrame(cols)
-
     def project_step(self, op, *, data_map, narrow):
         if op.node_name != "ProjectNode":
             raise TypeError("op was supposed to be a data_algebra.data_ops.ProjectNode")
@@ -265,23 +306,13 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
                 cols[k] = vk
         else:
             cols = {"_data_table_temp_col": res["_data_table_temp_col"].agg("sum")}
-
         # agg can return scalars, which then can't be made into a self.pd.DataFrame
-        def promote_scalar(v):
-            # noinspection PyBroadException
-            try:
-                len(v)
-            except Exception:
-                return [v]
-            return v
-
-        cols = {k: promote_scalar(v) for (k, v) in cols.items()}
-        res = self.columns_to_frame(cols)
+        res = self.columns_to_frame_(cols)
         res = res.reset_index(drop=len(op.group_by) < 1)  # grouping variables in the index
         missing_group_cols = set(op.group_by) - set(res.columns)
         assert len(missing_group_cols) <= 0
         if "_data_table_temp_col" in res.columns:
-            res = res.drop("_data_table_temp_col", 1)
+            res = res.drop("_data_table_temp_col", axis=1, inplace=False)
         # double check shape is what we expect
         if not data_algebra.util.table_is_keyed_by_columns(res, op.group_by):
             raise ValueError("result wasn't keyed by group_by columns")
@@ -295,6 +326,8 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         res = op.sources[0].eval_implementation(
             data_map=data_map, data_model=self, narrow=narrow
         )
+        if res.shape[0] < 1:
+            return res
         selection = op.expr.evaluate(res)
         res = res.loc[selection, :].reset_index(drop=True, inplace=False)
         return res
@@ -328,12 +361,15 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         res = op.sources[0].eval_implementation(
             data_map=data_map, data_model=self, narrow=narrow
         )
-        ascending = [
-            False if ci in set(op.reverse) else True for ci in op.order_columns
-        ]
-        res = res.sort_values(by=op.order_columns, ascending=ascending).reset_index(
-            drop=True
-        )
+        if res.shape[0] > 1:
+            ascending = [
+                False if ci in set(op.reverse) else True for ci in op.order_columns
+            ]
+            res = res.sort_values(by=op.order_columns, ascending=ascending).reset_index(
+                drop=True
+            )
+        if (op.limit is not None) and (res.shape[0] > op.limit):
+            res = res.iloc[range(op.limit), :].reset_index(drop=True)
         return res
 
     def rename_columns_step(self, op, *, data_map, narrow):
@@ -402,7 +438,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             if c not in op.by:
                 is_null = res[c].isnull()
                 res.loc[is_null, c] = res.loc[is_null, c + "_tmp_right_col"]
-                res = res.drop(c + "_tmp_right_col", axis=1)
+                res = res.drop(c + "_tmp_right_col", axis=1, inplace=False)
         res = res.reset_index(drop=True)
         return res
 
@@ -423,8 +459,12 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         type_checks = data_algebra.util.check_columns_appear_compatible(left, right)
         if type_checks is not None:
             raise ValueError(f"concat: incompatible column types: {type_checks}")
+        if left.shape[0] < 1:
+            return right
+        if right.shape[0] < 1:
+            return left
         # noinspection PyUnresolvedReferences
-        res = self.pd.concat([left, right], axis=0, ignore_index=True)
+        res = self.pd.concat([left, right], axis=0, ignore_index=True, sort=False)
         res = res.reset_index(drop=True)
         return res
 
